#!/usr/bin/ksh
###############################################################################
##                                                                           ##
##  D B T U T O O L S / D B T U S W I T C H . S H                            ##
##                                                                           ##
## ------------------------------------------------------------------------- ##
##                                                                           ##
##  Job .......: Script for changing the environmental settings based        ##
##               on the given instance name.                                 ##
##                                                                           ##
## ------------------------------------------------------------------------- ##
##                                                                           ##
##  Copyright (C) 2002  Daniel Scheibli                                      ##
##                                                                           ##
##  This program is free software; you can redistribute it and/or modify it  ##
##  under the terms of the GNU General Public License as published by the    ##
##  Free Software Foundation; either version 2 of the License, or (at your   ##
##  option) any later version.                                               ##
##                                                                           ##
##  This program is distributed in the hope that it will be useful, but      ##
##  WITHOUT ANY WARRANTY; without even the implied warranty of               ##
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            ##
##  GNU General Public License for more details.                             ##
##                                                                           ##
##  You should have received a copy of the GNU General Public License along  ##
##  with this program; if not, write to the Free Software Foundation, Inc.,  ##
##  59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.                 ##
##                                                                           ##
## ------------------------------------------------------------------------- ##
##                                                                           ##
##  Remark ....: <none>                                                      ##
##                                                                           ##
## ------------------------------------------------------------------------- ##
##                                                                           ##
##  Author ....: Daniel Scheibli <daniel.scheibli@edelbyte.org>              ##
##  Date ......: 2002-07-22                                                  ##
##  Changes ...: ----- dbtuswitch.sh V00.99/R020 --------------------------- ##
##               <none>                                                      ##
##               ----- dbtuswitch.sh V00.99/R011 --------------------------- ##
##               2002-11-23 - Re-adjust configuration file parsing based     ##
##                            on the modified format.                        ##
##               2002-11-07 - The root path of the /opt/dbtutools tree is    ##
##                            now set in one global variable.                ##
##               2002-11-02 - FuncActionSwitch() does an cleanup of the      ##
##                            environment variables ($PATH,...) to ensure    ##
##                            that there are no conflicts with settings      ##
##                            from the former DB2 instance.                  ##
##                          - FuncActionList() doesn't use db2ilist any      ##
##                            more. This way we can handle DB2 instances     ##
##                            from different versions.                       ##
##               ----- dbtuswitch.sh V00.99/R010 --------------------------- ##
##               <none>                                                      ##
##                                                                           ##
###############################################################################





##### Presettings (global)
#------------------------------------------------------------------------------
XX1OPTREEROOT="/opt/dbtutools"     ##### REMARK: No trailing slash allowed!
#------------------------------------------------------------------------------





###############################################################################
##                                                                           ##
##   F U N C A C T I O N L I S T                                             ##
##                                                                           ##
###############################################################################
XX1FuncActionList() {
 ##### Presettings
 #-----------------------------------------------------------------------------
 XX1INSTANCE=""
 XX1LIST=""
 #----------------------------------------------------------------------------- 



 ##### Try to read the name of the default instance
 #-----------------------------------------------------------------------------
 XX1INSTANCE=$(grep "^DefaultInstance=" $XX1OPTREEROOT/dbtuswitch.cfg 2>/dev/null | cut -d"=" -f2 | tr -d " " | tr -d "\t")
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 if [ -z "$XX1INSTANCE" ]; then
  echo "===> WARNING: The configuration file contains no 'DefaultInstance' value."; echo
 fi
 #-----------------------------------------------------------------------------



 ##### Try to read the list of instances (without the db2ilist tool)
 #-----------------------------------------------------------------------------
 ##### Read the list directly from its "source"
 XX1LIST=$(cat /var/db2/v*/profiles.reg 2>/dev/null)
 ##### If we have a list, we need to remove the admin server instance(s)
 if [ ! -z "$XX1LIST" ]; then
  ##### Read the name of the admin server instance(s)
  XX1DUMMY=$(grep "^DB2ADMINSERVER=" /var/db2/v*/default.env 2>/dev/null | cut -d"=" -f2 | cut -d"'" -f2)   ##### REMARK: Just for syntax highlighting: ')
  ##### Loop over the name(s) of all admin server instance(s)
  for I in $XX1DUMMY
  do
   ##### Modify the list of instance names
   XX1LIST=$(echo $XX1LIST | tr " " "\n" | grep -v "^$I$")
  done
 fi
 #----------------------------------------------------------------------------- 



 ##### Ensure, that we have a list of instance names
 #-----------------------------------------------------------------------------
 if [ -z "$XX1LIST" ]; then
  echo "===> ERROR: Reading the list of DB2 instances failed."
  XX1RC=1; return    
 fi
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 XX1LIST=$(echo $XX1LIST | tr " " "\n" | grep -v "^$" | sort)
 #----------------------------------------------------------------------------- 



 ##### Print the list of instances
 #-----------------------------------------------------------------------------
 echo "          Instance"
 echo "Status    Name     Comment" 
 echo "--------- -------- --------------------------------------------------"
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 for I in $XX1LIST
 do
  ##### Assemble the output line
  XX1DUMMY="-"
  if [ "$I" == "$XX1INSTANCE" ]; then
   XX1DUMMY="<default>"
  fi
  if [ "$I" == "$DB2INSTANCE" ]; then
   XX1DUMMY="<current>"
  fi
  printf "%-9s %-8s" "$XX1DUMMY" "$I"
  XX1DUMMY=$(grep -i "^Instance=$I:Comment=" $XX1OPTREEROOT/dbtuswitch.cfg 2>/dev/null | cut -d":" -f2- | cut -d"=" -f2-)
  if [ ! -z "$XX1DUMMY" ]; then
   printf " %-50s\n" "$XX1DUMMY"
  else
   printf " -\n"
  fi
 done
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 echo
 #----------------------------------------------------------------------------- 
}





###############################################################################
##                                                                           ##
##   F U N C A C T I O N S W I T C H                                         ##
##                                                                           ##
###############################################################################
XX1FuncActionSwitch() {
 ##### Presettings
 #-----------------------------------------------------------------------------
 XX1INSTANCEOLD=$DB2INSTANCE 
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 XX1INSTANCE=$(echo $XX1INSTANCE | tr "[A-Z]" "[a-z]")
 #----------------------------------------------------------------------------- 



 ##### Ensure, that we are running without X-Attribute
 #----------------------------------------------------------------------------- 
 if [ -x $XX1OPTREEROOT/dbtuswitch ]; then
  echo "===> ERROR: The X-Attribut is not allowed for this script."
  XX1RC=1; return    
 fi
 #-----------------------------------------------------------------------------   



 ##### React on a keyword as instance name ("current" or "default")
 #----------------------------------------------------------------------------- 
 ##### On "current" keyword,...
 if [ "$XX1INSTANCE" == "current" ]; then
  if [ -z "$DB2INSTANCE" ]; then
   echo "===> ERROR: The name of the current instance is not detectable."
   XX1RC=1; return
  else
   XX1INSTANCE=$DB2INSTANCE
  fi
 fi
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 ##### On "default" keyword,...
 if [ "$XX1INSTANCE" == "default" ]; then
  XX1INSTANCE=$(grep "^DefaultInstance=" $XX1OPTREEROOT/dbtuswitch.cfg 2>/dev/null | cut -d"=" -f2 | tr -d " " | tr -d "\t")
  if [ -z "$XX1INSTANCE" ]; then
   echo "===> ERROR: The configuration file contains no 'DefaultInstance' value."
   XX1RC=1; return
  fi
 fi
 #----------------------------------------------------------------------------- 



 ##### Locate the home directory of the instance owners (current and next instance)
 #----------------------------------------------------------------------------- 
 ##### For the current instance (we are going to leave)
 if [ ! -z "$XX1INSTANCEOLD" ]; then
  XX1PATHOLD=$(grep "^$XX1INSTANCEOLD:" /etc/passwd 2>/dev/null | cut -d":" -f6)
  if [ -z "$XX1PATHOLD" ]; then
   echo "===> ERROR: The instance owners home directory could not be located (1)."
   XX1RC=1; return
  fi  
 else
  XX1PATHOLD=""
 fi
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 ##### For the next instance (we are going to switch to)
 XX1PATH=$(grep "^$XX1INSTANCE:" /etc/passwd 2>/dev/null | cut -d":" -f6)
 if [ -z "$XX1PATH" ]; then
  echo "===> ERROR: The instance owners home directory could not be located (2)."
  XX1RC=1; return
 fi  
 #-----------------------------------------------------------------------------  



 ##### Ensure, that the db2profile script exists and call it
 ##### (in between it is needed to cleanup the environment variables
 ##### from the old settings - something db2profile partly did, but
 ##### even with the UDB Version 8.1 Beta there seems to be some
 ##### problems)
 #----------------------------------------------------------------------------- 
 XX1DUMMY=$XX1PATH"/sqllib/db2profile"
 if [ ! -f "$XX1DUMMY" ]; then
  echo "===> ERROR: The db2profile script could not be located."
  XX1RC=1; return 
 fi
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 XX1FuncPathListCleanup "$PATH"            "$XX1PATHOLD" 1; export PATH=$XX1LIST
 XX1FuncPathListCleanup "$LD_LIBRARY_PATH" "$XX1PATHOLD" 1; export LD_LIBRARY_PATH=$XX1LIST
 XX1FuncPathListCleanup "$CLASSPATH"       "$XX1PATHOLD" 1; export CLASSPATH=$XX1LIST
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - 
 XX1DUMMY=$XX1PATH"/sqllib/db2profile"
 . $XX1DUMMY
 if [ "$?" != "0" ]; then
  echo "===> ERROR: Call to the db2profile script returned with error."
  XX1RC=1; return 
 fi  
 #----------------------------------------------------------------------------- 



 ##### Set up the proper DB2 node (if given)
 #----------------------------------------------------------------------------- 
 if [ ! -z "$XX1NODE" ]; then
  DB2NODE=$XX1NODE
 fi
 #----------------------------------------------------------------------------- 



 ##### Do a terminate to ensure that there are no
 ##### conflicts caused by an old background process.
 #----------------------------------------------------------------------------- 
 db2 terminate 1>/dev/null
 if [ "$?" != "0" ]; then
  echo "===> ERROR: 'db2 TERMINATE' call returned with error."
  XX1RC=1; return 
 fi  
 #----------------------------------------------------------------------------- 



 ##### Modify environmental settings for DbtuTools (ease of use)
 #----------------------------------------------------------------------------- 
 XX1FuncPathListCleanup "$PATH" "$XX1OPTREEROOT/" 0
 if [ -z "$XX1DUMMY" ]; then
  export PATH=$XX1LIST""$XX1OPTREEROOT"/"$XX1INSTANCE"/tools/bin"
 else
  export PATH=$XX1LIST""$XX1OPTREEROOT"/"$XX1INSTANCE"/tools/bin:." 
 fi
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 XX1FuncPathListCleanup "$CLASSPATH" "$XX1OPTREEROOT/" 0
 if [ -z "$XX1DUMMY" ]; then
  export CLASSPATH=$XX1LIST""$XX1OPTREEROOT"/"$XX1INSTANCE"/tools/bin/dbtutools.jar"
 else
  export CLASSPATH=$XX1LIST""$XX1OPTREEROOT"/"$XX1INSTANCE"/tools/bin/dbtutools.jar:."
 fi
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 export DBTUTOOLSHOME=$XX1OPTREEROOT"/"$XX1INSTANCE
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 alias dbtuxconnect.sh=". dbtuxconnect.sh"
 alias dbtuswitch.sh=". $XX1OPTREEROOT/dbtuswitch.sh"
 #----------------------------------------------------------------------------- 



 ##### Announce successfull switch to the given DB2 instance 
 #----------------------------------------------------------------------------- 
 if [ -z "$XX1NODE" ]; then
  echo "Switch to instance '"$XX1INSTANCE"' successfully done."; echo
 else
  echo "Switch to instance '"$XX1INSTANCE"' and node '"$XX1NODE"' successfully done."; echo
 fi
 #----------------------------------------------------------------------------- 
}





###############################################################################
##                                                                           ##
##   F U N C A C T I O N U S A G E                                           ##
##                                                                           ##
###############################################################################
XX1FuncActionUsage() {
 ##### Print syntax diagram
 #-----------------------------------------------------------------------------
 echo "===> ERROR: A minimum of one argument was expected."
 echo
 echo "Usage .......: >>--- . dbtuswitch.sh ---+--- INSTANCE ---+--->"
 echo "                                        |                |"  
 echo "                                        +--- default ----+"
 echo "                                        |                |"
 echo "                                        +--- current ----+" 
 echo
 echo "                >---+------------+---------------------------><" 
 echo "                    |            |"
 echo "                    +--- NODE ---+"
 echo
 echo "Sample ......: . dbtuswitch.sh db2inst1"
 echo
 echo "Arguments ...: INSTANCE   Name of the DB2 instance to switch to or one of the"
 echo "                          following two keywords:"
 echo "                          - 'default' = Switch to the default instance - set" 
 echo "                                        in "$XX1OPTREEROOT"/dbtuswitch.cfg file."
 echo "                          - 'current' = Switch to the instance named in the"
 echo "                                        \$DB2INSTANCE environment variable."
 echo "               NODE       Node number to be set in the \$DB2NODE environment"
 echo "                          variable." 
 echo
 #-----------------------------------------------------------------------------
}





###############################################################################
##                                                                           ##
##   F U N C P A T H L I S T C L E A N U P                                   ##
##                                                                           ##
###############################################################################
XX1FuncPathListCleanup() {
 ##### Cut out $2 from $1
 ##### TODO: Solve the problem using sed will be more effective?
 #-----------------------------------------------------------------------------
 XX1LIST=$(echo $1 | tr ":" "\n" | grep -v "^$2" | grep -v "^$" | grep -v "^\.$" | tr "\n" ":")
 XX1DUMMY=$(echo $1 | tr ":" "\n" | grep -v "^\.$")
 if [ ! -z "$XX1DUMMY" ]; then
  if [ "$3" != "0" ]; then
   XX1LIST=$XX1LIST":."
  fi
 fi
 #-----------------------------------------------------------------------------
}





###############################################################################
##                                                                           ##
##   F U N C F I N I                                                         ##
##                                                                           ##
###############################################################################
XX1FuncFini() {
 ##### Unset all variables and functions used in the script
 #-----------------------------------------------------------------------------
 unset XX1OPTREEROOT
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 unset XX1ARGS XX1ARG1 XX1ARG2
 unset XX1INSTANCE XX1INSTANCEOLD XX1NODE
 unset XX1PATH XX1PATHOLD
 unset XX1LIST 
 unset XX1DUMMY
 #- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 unset XX1FuncActionList XX1FuncActionSwitch XX1FuncActionUsage
 unset XX1FuncPathListCleanup
 #-----------------------------------------------------------------------------
}





###############################################################################
##                                                                           ##
##   < M A I N >                                                             ##
##                                                                           ##
###############################################################################



##### Presettings
#------------------------------------------------------------------------------
echo; echo "DBTUSWITCH.SH V00.99/R020"; echo
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - --
XX1RC=0
#------------------------------------------------------------------------------



##### Process the given arguments
#------------------------------------------------------------------------------
XX1ARGS=$#; XX1ARG1=$1; XX1ARG2=$2
while (($#>0))
do
 shift
done
#- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - --
case $XX1ARGS in
 0) ##### List DB2 instances
    XX1FuncActionList
    ;;
 1) ##### Switch to DB2 instance
    XX1INSTANCE=$XX1ARG1
    XX1NODE=""
    XX1FuncActionSwitch
    ;;
 2) ##### Switch to DB2 instance and node
    XX1INSTANCE=$XX1ARG1
    XX1NODE=$XX1ARG2
    XX1FuncActionSwitch
    ;;
 *) ##### Print Usage
    XX1FuncActionUsage
    XX1RC=1
    ;;
esac
#------------------------------------------------------------------------------



##### End program / script
#------------------------------------------------------------------------------
XX1FuncFini; unset XX1FuncFini
case $XX1RC in
 0) ##### On RC = 0,...
    unset XX1RC; return 0
    ;;
 *) ##### On any other RC,... 
    unset XX1RC; return 1 
    ;;
esac    
#------------------------------------------------------------------------------




