/* ######################################################################### */
/* ##                                                                     ## */
/* ##  D B T U T O O L S / C O R E / O S / T E M P F I L E . J A V A      ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Job .......: Class, for creating temporary files within the        ## */
/* ##               /opt/dbtutools tree.                                  ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Copyright (C) 2002  Daniel Scheibli                                ## */
/* ##                                                                     ## */
/* ##  This library is free software; you can redistribute it and/or      ## */
/* ##  modify it under the terms of the GNU Lesser General Public         ## */
/* ##  License as published by the Free Software Foundation; either       ## */
/* ##  version 2.1 of the License, or (at your option) any later version. ## */
/* ##                                                                     ## */
/* ##  This library is distributed in the hope that it will be useful,    ## */
/* ##  but WITHOUT ANY WARRANTY; without even the implied warranty of     ## */
/* ##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU  ## */
/* ##  Lesser General Public License for more details.                    ## */
/* ##                                                                     ## */
/* ##  You should have received a copy of the GNU Lesser General Public   ## */
/* ##  License along with this library; if not, write to the Free         ## */
/* ##  Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,     ## */
/* ##  MA 02111-1307 USA                                                  ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Remark ....: <none>                                                ## */
/* ##                                                                     ## */
/* ## ------------------------------------------------------------------- ## */
/* ##                                                                     ## */
/* ##  Author ....: Daniel Scheibli <daniel.scheibli@edelbyte.org>        ## */
/* ##  Date ......: 2002-10-06                                            ## */
/* ##  Changes ...: ----- TempFile.java V00.99/R020 --------------------- ## */
/* ##               <none>                                                ## */
/* ##               ----- TempFile.java V00.99/R011 --------------------- ## */
/* ##               <none>                                                ## */
/* ##                                                                     ## */
/* ######################################################################### */



/* ##### Presettings */
/* ------------------------------------------------------------------------- */
package org.dbtutools.DbtuTools.core.os;
/* ------------------------------------------------------------------------- */





/* ##### Class definition */
/** ************************************************************************
 * This class should be used to create temporary files
 * within the /opt/dbtutools tree. It cannot be instantiated. 
 * @author Daniel Scheibli &lt;daniel.scheibli at edelbyte.org&gt;
 * ************************************************************************* */
public class TempFile
{
 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   T E M P F I L E ()                                               ## */
 /* ##                                                                    ## */
 /* ######################################################################## */ 
 /** ***********************************************************************
  * Private declared Constructor, preventing the instantiation
  * of the <code>TempFile</code> class.
  * ************************************************************************ */
 private TempFile(  )
 { }
 
 
 
 
 
 /* ######################################################################## */
 /* ##                                                                    ## */
 /* ##   C R E A T E T E M P F I L E ()                                   ## */ 
 /* ##                                                                    ## */
 /* ######################################################################## */ 
 /** ***********************************************************************
  * Creates a File class instance, pointing to a newly created
  * (empty) temporary file, which is located in the /opt/dbtutools
  * tree.<br />
  * <br />
  * <i>Remark:</i> The name of the temporary file is based on the
  * following format:
  * <code>t&lt;yyyymmddhhmmss&gt;p&lt;nnnn&gt;n&lt;n&gt;.tmp</code>
  * The first token contains time and date; the next token represents
  * the parent process id. The last token is an additional number 
  * (most of the times zero) which is increased in a case of a
  * conflicts with an already existing file.
  *
  * @return The File instance, addressing the created temporary file.
  * @exception org.edelbyte.util.UnknownValueException
  *            For the case, that the /opt/dbtutools tree or the
  *            parent process id could not be detected.
  * @exception java.io.IOException
  *            For the case, that the temporary file could not
  *            be created (there are different reasons for).
  * ************************************************************************ */
 public static java.io.File createTempFile( ) throws org.edelbyte.util.UnknownValueException,
                                                     java.io.IOException
 {
  /* ##### Presettings */
  /* ----------------------------------------------------------------------- */
  java.io.File               oFile          = null;
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  java.lang.String           oPathNameToken = org.dbtutools.DbtuTools.core.OptDbtuToolsTree.getInstance().getPathNames().get("Tree.Instance.Temp").toString();
  int                        iParentPid     = org.dbtutools.DbtuTools.core.os.System.getParentProcessId();
  java.lang.StringBuffer     oFileName;
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  java.util.Date             oDate          = java.util.Calendar.getInstance().getTime();
  java.text.SimpleDateFormat oDateFormatter = new java.text.SimpleDateFormat("yyyyMMddHHmmss");
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  boolean                    bCallRc;
  int                        I;
  /* ----------------------------------------------------------------------- */



  /* ##### Try to create our temporary file */
  /* ----------------------------------------------------------------------- */
  for( I = 0; I < 10; I++ )
  {
   /* ##### Assemble the file name */
   oFileName = new java.lang.StringBuffer();
   oFileName.append( oPathNameToken ).append( java.io.File.separator       );
   oFileName.append( "t"            ).append( oDateFormatter.format(oDate) );
   oFileName.append( "p"            ).append( iParentPid                   );
   oFileName.append( "n"            ).append( I                            );
   oFileName.append( ".tmp"         );

   /* ##### Try to create the new file */
   oFile   = new java.io.File( oFileName.toString() );
   bCallRc = oFile.createNewFile();
   /* ##### On success,... */
   if( bCallRc == true )
   { break; }

   /* ##### Release objects (helping the garbage collector) */
   oFileName = null;
   oFile     = null;
  }
  /* ----------------------------------------------------------------------- */



  /* ##### Return result */
  /* ----------------------------------------------------------------------- */
  if( oFile != null )
  { return( oFile ); }
  /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
  else
  { throw new java.io.IOException("All tried file names still exists."); }
  /* ----------------------------------------------------------------------- */
 }
}
/* ------------------------------------------------------------------------- */


